@@ -20,10 +20,10 @@ def backwards_replace(
     """
     expr = copy.deepcopy(expr)
     if isinstance(expr, ir.Var):
-        if not isinstance(expr, ir.ExclusiveVar):
-            for key in mapping:
-                if key.to_string() == expr.to_string():
-                    return mapping[key]
+        # if not isinstance(expr, ir.ExclusiveVar):
+        for key in mapping:
+            if key.to_string() == expr.to_string():
+                return mapping[key]
     elif isinstance(expr, (ir.UInt, ir.Int)):
         return expr
     elif isinstance(expr, (ir.BinOp, ir.UBinOp)):
